## Set Working Directory
setwd("C:/users/piccolobriand/documents/acnc/Kartik/CTSA Western States Collab Pilot 2015/Oxylipids")


install_packages <- c("readxl","tidyverse", "outliers","car","pls", "caret","boot","flextable","rgl")
install_BioCpackages <- "impute"
## Check library for packages and then install from CRAN if necessary
if (length(setdiff(install_packages, rownames(installed.packages()))) > 0) {
    install.packages(setdiff(install_packages, rownames(installed.packages())))
}
## Check library for packages and then install from Bioconductor if necessary
if (length(setdiff(install_BioCpackages, rownames(installed.packages()))) > 0) {
	BiocManager::install(setdiff(install_BioCpackages, rownames(installed.packages())), ask = FALSE)
}

## libraries and in-house functions
library(readxl)
library(tidyverse)
library(outliers)
library(impute)
library(car) 
library(pls) 
library(caret) 
library(boot) 
library(flextable)
library(rgl)


grubbsOUT_FUN <- function(DAT, trans, cutoff) {
	## transform
	transDAT <- switch(trans,
		"none" = DAT,
		"log" = log(DAT),
		"sqrt" = sqrt(DAT)
	)		
	## Run Grubb's test
	GT <- grubbs.test(transDAT, type=10)	
	## Make new data object
	transDATcut <- transDAT	
	## If GT pvalue is > cutoff, return new data
	if(GT$p.value > cutoff) {
		transDATcut
	## if pvalue is < cutoff
	} else {
		## Make GT pvalue object
		gtp <- GT$p.value
		## use while loop to iteratively identify outlier, remove, rerun GT, and evaluate p value.
		## Will stop when GT pvalue > set cutoff
		while(gtp < cutoff) {
			## Identify single outlier
			gtoutlier <- outliers::outlier(transDATcut)	
			## Find position of outlier in data
			outpos <- which(transDATcut %in% gtoutlier)
			## Change outlier to NA
			transDATcut[outpos] <- NA
			## Rerun GT and extract p value
			gtp <- grubbs.test(transDATcut, type=10)$p.value			
		}
		transDATcut	
	}
}	

plotat <- function(RANGE) {
	if(length(RANGE) != 2) stop("RANGE argument must have a length of 2")
	if(RANGE[1] > RANGE[2]) stop("First element in RANGE must be smaller then second element")
	prettyres <- pretty(sprintf("%.2f",RANGE[1]):sprintf("%.2f",RANGE[2]), 7)
	while((min(prettyres) < RANGE[1]) == FALSE) {
		prdiff <- prettyres[2] - prettyres[1]
		prettyres[length(prettyres) + 1] <- prettyres[1] - prdiff
		prettyres <- sort(prettyres)
	} 
	while((max(prettyres) > RANGE[2]) == FALSE) {
		prdiff <- prettyres[2] - prettyres[1]
		prettyres[length(prettyres) + 1] <- prettyres[length(prettyres)] + prdiff
		prettyres <- sort(prettyres)	
	}	
	plotticks <- as.numeric(sprintf("%.2f",prettyres))
	plotticks
}

## ellipseplot function
ellipseplot <- function(x, y, factr, 
						elev=0.95, # Ellipse probability level
						legpos=c("topright","topleft","bottomleft","bottomleft"), # Legend position
					    pcol=NULL, # manual addition of colors, must meet length of factors
						cexsize=1, # point size
						ppch=21, # Point type, must meet length of factors
						legcexsize=2, # legend font size
						legptsize=2, # legend point size
						pbgcol=TRUE,
						axissize=1, 
						linewidth=1, 
						font=1) {
	## Set factor levels
	if(is.factor(factr)) {
		f <- factr
	} else {
		f <- factor(factr, levels=unique(as.character(factr)))
	}
	intfactr <- as.integer(f) # Set integer vector that matches factor levels
	# Checking to make sure length of ppch equals number of factor levels
	if((length(ppch) > 1 & length(unique(intfactr)) != length(ppch))) stop("Can only increase point shape if equal to factor levels")
	# pch <- unique(ppch)[as.integer(f)]

	edf <- data.frame(LV1 = x, LV2=y, factr = f) # create data frame with data and factor
	
	ellipses <- (edf %>% group_by(factr) %>% do(mod = dataEllipse(.$LV1, .$LV2, levels=0.95, robust=TRUE, draw=FALSE)))$mod

	## Get range of x and y data
	xrange <- plotat(range(c(as.vector(sapply(ellipses, function(x) x[,1])), min(x), max(x))))
	yrange <- plotat(range(c(as.vector(sapply(ellipses, function(x) x[,2])), min(y), max(y))))
	
	if(is.null(pcol) != TRUE) { # If colors are supplied by user
		ptcol <- pcol
		pgcol <- paste(pcol, "7e", sep="") # adds opaqueness
	} else { # Default
		pgcol <- c("#e41a1c7e","#377eb87e","#4daf4a7e","#984ea37e","#807f7d7e") # Defaults at 5 colors
		ptcol <- c("#e41a1c","#377eb8","#4daf4a","#984ea3","#807f7d") # For opaqueness
	}
	# Plotting graphic
	plot(x,y, type="n", xlab="", ylab="", main="", xlim=range(xrange), ylim=range(yrange), axes=FALSE)
	axis(1, at=xrange, labels=xrange, cex.axis=axissize,lwd=linewidth, font=font)
	axis(2, las=2, cex.axis=axissize,lwd=linewidth, font=font)
	graphics::box(lwd=linewidth, font=font)
	abline(h=0, v=0, col="gray", lty=2) # Adds lines at 0
	legpch <- c() # vector to collect legend pch data
	legcol <- c() # vector to collect legend col data
	## Not sure why I split this up, might have been an artifact of an older version.
	## Adds points, ellipse, and determines color specifications for legend 
	if(pbgcol==TRUE)  {
		for(i in 1:length(unique(intfactr))){
			polygon(ellipses[[i]], col=pgcol[i], border=ptcol[i])
			points(x[intfactr==i], y[intfactr==i], pch=ppch[i], col=ptcol[i], bg=ptcol[i],cex=cexsize)
			legpch[i] <- ppch[i]
			legcol[i] <- ptcol[i]
		}
	} else {
		for(i in 1:length(unique(intfactr))){
			polygon(ellipses[[i]], col=pgcol[i], border=ptcol[i])
			points(x[intfactr==i], y[intfactr==i], pch=ppch[i], col="black", bg=ptcol[i],cex=cexsize)
			legpch[i] <- ppch[i]
			legcol[i] <- ptcol[i]		
		}
	}
	## Legend
	if(pbgcol==TRUE) {
		graphics::legend(x=legpos, legend=levels(f), pch=legpch, 
			pt.bg=legcol, col=legcol, bty="n", border=FALSE, pt.cex=legptsize, cex=legcexsize)
	} else {
		graphics::legend(x=legpos, legend=levels(f), pch=legpch, 
			pt.bg=legcol, col="black", bty="n", border=FALSE, pt.cex=legptsize, cex=legcexsize)	
	}
}	

## Axis legends for PCA output using prcomp() function
PCAvarAxis <- function(PCA, decimal=1) {
	pcavar <- round((PCA$sdev^2)/sum((PCA$sdev^2)),3)*100   #Calculate % variance explained
	PC1var <- paste("Principal Component 1 (", pcavar[1], "%)", sep="")
	PC2var <- paste("Principal Component 2 (", pcavar[2], "%)", sep="")
	PC3var <- paste("Principal Component 3 (", pcavar[3], "%)", sep="")
	PC4var <- paste("Principal Component 4 (", pcavar[4], "%)", sep="")	
	PC5var <- paste("Principal Component 5 (", pcavar[5], "%)", sep="")		
	return(list(PC1=PC1var, PC2=PC2var, PC3=PC3var, PC4=PC4var, PC5=PC5var))
}	

## Axis legends for pls output from pls pacakge	
PLSvarAxis <- function(PLSDA, decimal=1) {
	Yvar <- round(100 * drop(pls::R2(PLSDA, estimate = "train",intercept = FALSE)$val),decimal ) # Deterime % variance explained in Y-space
	## plsda with > 2 groups will return a matrix.  Variance is explained for each group, so a total explained variance is not provided
	if(is.matrix(Yvar)) { # plsda with > 2 groups
		warning("Cannot produce explained variance for Y space with > 2 classification groups")
		Xvar <- round(100 * PLSDA[["Xvar"]]/PLSDA[["Xtotvar"]],decimal ) ## Calculate variance in X-space
		YLV1var <- "Latent Variable 1"
		YLV2var <- "Latent Variable 2"
		YLV3var <- "Latent Variable 3"
		YLV4var <- "Latent Variable 4"
		XLV1var <- paste("Latent Variable 1 (", Xvar[1], "%)", sep="")
		XLV2var <- paste("Latent Variable 2 (", Xvar[2], "%)", sep="")
		XLV3var <- paste("Latent Variable 3 (", Xvar[3], "%)", sep="")
		XLV4var <- paste("Latent Variable 4 (", Xvar[4], "%)", sep="")
	} else { # plsda with only 2 groups
		YvarT <- round(c(Yvar[1], Yvar[2]-Yvar[1], Yvar[3]-Yvar[2], Yvar[4]-Yvar[3]), decimal) ## Calculate variance in Y-space
		YLV1var <- paste("Latent Variable 1 (", YvarT[1], "%)", sep="")
		YLV2var <- paste("Latent Variable 2 (", YvarT[2], "%)", sep="")
		YLV3var <- paste("Latent Variable 3 (", YvarT[3], "%)", sep="")
		YLV4var <- paste("Latent Variable 4 (", YvarT[4], "%)", sep="")
		Xvar <- round(100 * PLSDA[["Xvar"]]/PLSDA[["Xtotvar"]],decimal ) ## Calculate variance in X-space
		XLV1var <- paste("Latent Variable 1 (", Xvar[1], "%)", sep="")
		XLV2var <- paste("Latent Variable 2 (", Xvar[2], "%)", sep="")
		XLV3var <- paste("Latent Variable 3 (", Xvar[3], "%)", sep="")
		XLV4var <- paste("Latent Variable 4 (", Xvar[4], "%)", sep="")
	}
	return(list(XLV1=XLV1var, XLV2=XLV2var, XLV3=XLV3var, XLV4=XLV4var, 
		YLV1=YLV1var, YLV2=YLV2var,YLV3=YLV3var, YLV4=YLV4var))
	
}	


outofbagSCALE <- function(OOB, INB, scaleMETHOD){  
	if(is.null(ncol(OOB))) {
		if(!is.null(ncol(INB))) stop("There was a single vector and a matrix/data frame provided to function")
		if(length(OOB) != length(INB)) stop("The two vectors are of differing lengths")
		switch(scaleMETHOD,
			"auto" = (OOB - mean(INB, na.rm=TRUE)) / sd(INB, na.rm=TRUE),
			"pareto" = (OOB - mean(INB, na.rm=TRUE)) / sqrt(sd(INB, na.rm=TRUE)),
			"range" = (OOB - mean(INB, na.rm=TRUE)) / (max(INB, na.rm=TRUE) - min(INB, na.rm=TRUE)),
			"vast" = ((OOB - mean(INB, na.rm=TRUE))/sd(INB, na.rm=TRUE)) * (mean(INB, na.rm=TRUE)/sd(INB, na.rm=TRUE)),
			"level" = (OOB - mean(INB, na.rm=TRUE))/mean(INB, na.rm=TRUE)
		)
	} else {
		if(is.null(ncol(INB))) stop("There was a single vector and a matrix/data frame provided to function")
						
		oobdim <- dim(OOB)
		inbdim <- dim(INB)

		if(oobdim[2] != inbdim[2]) stop("The two matrices have different column lengths")
		if(setequal(colnames(OOB) == colnames(INB), FALSE)) stop("The two matrices have different column labels")
		sapplylength <- 1:ncol(OOB)
		switch(scaleMETHOD,
			"auto" = sapply(sapplylength, function(x) (OOB[,x] - mean(INB[,x], na.rm=TRUE)) / sd(INB[,x], na.rm=TRUE)),
			"pareto" = sapply(sapplylength, function(x) (OOB[,x] - mean(INB[,x], na.rm=TRUE)) / sqrt(sd(INB[,x], na.rm=TRUE))),
			"range" = sapply(sapplylength, function(x) (OOB[,x] - mean(INB[,x], na.rm=TRUE)) / (max(INB[,x], na.rm=TRUE) - min(INB[,x], na.rm=TRUE))),
			"vast" = sapply(sapplylength, function(x) ((OOB[,x] - mean(INB[,x], na.rm=TRUE))/sd(INB[,x], na.rm=TRUE)) * (mean(INB[,x], na.rm=TRUE)/sd(INB[,x], na.rm=TRUE))),
			"level" = sapply(sapplylength, function(x) (OOB[,x] - mean(INB[,x], na.rm=TRUE))/mean(INB[,x], na.rm=TRUE))
		)
	}
					
					
}			
	
VIP <- function(object) {
    if (object$method != "oscorespls")
        stop("Only implemented for orthogonal scores algorithm.  Refit with 'method = \"oscorespls\"'")
    if (nrow(object$Yloadings) > 1)
        stop("Only implemented for single-response models")

    SS <- c(object$Yloadings)^2 * colSums(object$scores^2)
    Wnorm2 <- colSums(object$loading.weights^2)
    SSW <- sweep(object$loading.weights^2, 2, SS / Wnorm2, "*")
    sqrt(nrow(SSW) * apply(SSW, 1, cumsum) / cumsum(SS))
}

vip_bootFUN <- function(dat, ind, comps){
	plsdamod <- pls::mvr(Y~., data=dat, subset=ind, ncomp=comps, method="oscorespls")
	if(comps == 1) {
		VIP(plsdamod)
	} else {
		colMeans(VIP(plsdamod)[1:comps,])
	}
}

coef_bootFUN <- function(dat, ind, comps=10){
# select obs. in bootstrap sample
	plsdamod <- pls::mvr(Y~., data=dat, subset=ind, ncomp=comps, method="oscorespls")
	coef(plsdamod)

}

plsdaVIPboot_FUN <- function(DATA, nc=10, iter=1000, Ytype=c("Categorical", "Continuous")) { 
	Ytype <- match.arg(Ytype)
	DAT <- DATA
	Y <- DATA$Y
	cat("\nBootstrapping")
	## Bootstrap
	if(Ytype == "Categorical") {
		set.seed(1000)
		plsdaVIPboot <- boot(DAT, vip_bootFUN, iter, strata=as.factor(Y), comps=nc)
	} else {
		set.seed(1000)
		plsdaVIPboot <- boot(DAT, coef_bootFUN, iter, comps=nc)
	}
	CI_LIST <- list()
	cat("\nAssessing Confidence Intervals")
	## Confidence intervals
	## Will identify CI that cannot be computed and give a placeholder
	for(i in 1:length(plsdaVIPboot$t0)){
		tryCatch({
			CI_LIST[[i]] <- boot.ci(plsdaVIPboot, conf=0.95, type="all", index=i)
		}, error=function(e) { })
	}
	if(setequal(sapply(CI_LIST, is.null), FALSE)) {
		CI_LIST <- CI_LIST
	} else {
		NULLresults <- which(sapply(CI_LIST, is.null))
		for(i in NULLresults) {
			CI_LIST[[i]] <- boot.ci(plsdaVIPboot, conf=0.95, type=c("norm", "basic", "perc"), index=i)
		}
	}

	cat("\nFormatting CI data frame\n")
	## Identify null CI and set CI as 0
	plsdaboot_CI <- lapply(CI_LIST, function(x) {
				
		if(is.null(x[["normal"]])) {
			nCI <- c(0,0)
		} else {
			nCI <- x[["normal"]][1,2:3]
		}
		names(nCI) <- c("Lower", "Upper")
		if(is.null(x[["basic"]])) {
			bCI <- c(0,0)
		} else {
			bCI <- x[["basic"]][1,4:5]
		}
		names(bCI) <- c("Lower", "Upper")
		if(is.null(x[["percent"]])) {
			pCI <- c(0,0)
		} else {
			pCI <- x[["percent"]][1,4:5]
		}
		names(pCI) <- c("Lower", "Upper")
		if(is.null(x[["bca"]])) {
			bcaCI <- c(0,0)
		} else {
			bcaCI <- x[["bca"]][1,4:5]
		}
		names(bcaCI) <- c("Lower", "Upper")
		list(normal=nCI, basic=bCI, percent=pCI, bca=bcaCI)	
	})
	
	## Extract all CI calculations
	plsdaboot_CItypes <- lapply(c("normal", "basic", "percent", "bca"), function(x) {
		plsdaboot_CIsub <- as.data.frame(t(sapply(plsdaboot_CI, function(y) y[[x]])))
		if(Ytype == "Categorical") {
			rownames(plsdaboot_CIsub) <- names(plsdaVIPboot$t0)
		} else {
			rownames(plsdaboot_CIsub) <- rownames(plsdaVIPboot$t0)
		}
		plsdaboot_CIsub		
	})
	names(plsdaboot_CItypes) <- c("normal", "basic", "percent", "bca")
	list(BOOT=plsdaVIPboot, BOOTCI=plsdaboot_CItypes)

}		
	
## Import data
OXY_Excelraw <- read_xlsx("WCMC MX299277 Alzheimers Oxylipins.xlsx")

###########################################
## Extract Metabolite meta data.
## WCMC formats data for Excel and has sample metadata along first handful of rows and metabolite metadata in the inital columns
## I'm assuming WCMC does not prefix any labels with ..., so will isolate columns with ... prefixes.
## This leaves blank cells in the top left corner.
## Metabolite metadata includes columns labeled 'LOD' through 'formula'
## Will extract.

## Isolate rows that include metabolite metadata
OXY_METABMETADATA <- OXY_Excelraw[grep("LOD", OXY_Excelraw %>% pull("...1")):nrow(OXY_Excelraw),]
# OXY_METABMETADATA <- OXY_Excelraw[grep("LOD", OXY_Excelraw %>% pull("X__1")):nrow(OXY_Excelraw),]
## Isolate columns that include metabolite metadata
OXY_METABMETADATA <- OXY_METABMETADATA[,grep("LOD", OXY_METABMETADATA[1,]):grep("formula", OXY_METABMETADATA[1,])]
## Set column names with first row.
colnames(OXY_METABMETADATA) <- OXY_METABMETADATA[1,] %>% unlist()
## Remove first row
OXY_METABMETADATA <- OXY_METABMETADATA %>% filter(!(MRM %in% "MRM"))
## All columns are characters, so will coerce those with all numbers into numeric class and leave others as character.
OXY_METABMETADATA <- as.data.frame(lapply(OXY_METABMETADATA, function(x) {
	if(FALSE %in% !is.na(as.numeric(x))){
		y <- x
	} else {
		y <- as.numeric(x)
	}
	y
}), stringsAsFactors = FALSE)
## Import FA origin data
Agrawalmeta <- read.csv("Oxylipin Class and FA key from Agrawal 2017 j lipid res.csv", header=TRUE, comment.char = "", check.names = FALSE)
## Join data
OXY_METABMETADATA <- full_join(OXY_METABMETADATA, Agrawalmeta %>% dplyr::select(approved.short.names, classNOspace, Faprecursor), by="approved.short.names")

###########################################
## Extract Sample meta data
## Identify columns to retain.
dbpos <- grep("database filenames", colnames(OXY_Excelraw))
oxpos <- grep("Oxylipids", colnames(OXY_Excelraw))
## Subset data
OXY_SAMPLEMETADATA <- OXY_Excelraw[,c(dbpos, oxpos)]
## Remove measured data
OXY_SAMPLEMETADATA <- OXY_SAMPLEMETADATA[1:grep("organ", OXY_SAMPLEMETADATA %>% pull("database filenames")),]
## Remove empty columns
OXY_SAMPLEMETADATA <- OXY_SAMPLEMETADATA[,(which(colnames(OXY_SAMPLEMETADATA) %in% "database filenames")):ncol(OXY_SAMPLEMETADATA)]
## first and second row are the same.  Will add "database filenames" string to first cell in first row as it was not in raw file.
OXY_SAMPLEMETADATA[1,1] <- "database filenames"
## Coerce to data frame
OXY_SAMPLEMETADATA <- as.data.frame(OXY_SAMPLEMETADATA)
## Set rownames
rownames(OXY_SAMPLEMETADATA) <- OXY_SAMPLEMETADATA[,1]
## Remove first column
OXY_SAMPLEMETADATA <- OXY_SAMPLEMETADATA[,2:ncol(OXY_SAMPLEMETADATA)]
## Transpose and coerce back into data frame
OXY_SAMPLEMETADATA <- as.data.frame(t(OXY_SAMPLEMETADATA))

###########################################
## Extract Data
## Need "approved short names" column as metabolite labels and "label" row as sample labels.
OXY_DATA_df <- as.data.frame(OXY_Excelraw)
## Identify position of first sample label
sampLABstart <- which(colnames(OXY_DATA_df) %in% "database filenames") + 1
## Identify position of last sample name
sampLABsend <- ncol(OXY_DATA_df)
## Overwrite sample numbers with label names
OXY_DATA_df[grepl("LOD", OXY_DATA_df[,"...1"]),sampLABstart:sampLABsend] <- 
# OXY_DATA_df[grepl("LOD", OXY_DATA_df[,"X__1"]),sampLABstart:sampLABsend] <- 
	OXY_DATA_df[grepl("label", OXY_DATA_df[,colnames(OXY_DATA_df) %in% "database filenames"]),sampLABstart:sampLABsend]
## Remove rows with empty cells until columns with sample metadata
OXY_DATA_df <- OXY_DATA_df[!(OXY_DATA_df[,"...1"] %in% NA),]
# OXY_DATA_df <- OXY_DATA_df[!(OXY_DATA_df[,"X__1"] %in% NA),]
## Overwrite column names with first row
colnames(OXY_DATA_df) <- OXY_DATA_df[1,]
## Remove first row
OXY_DATA_df <- OXY_DATA_df[!(grepl("LOD", OXY_DATA_df[,grepl("LOD", colnames(OXY_DATA_df))])),]
## Set row names as "approved short names" column name
rownames(OXY_DATA_df) <- OXY_DATA_df[,"approved short names"]
## Extract data
OXY_DATA_df <- OXY_DATA_df[, grepl("RED", colnames(OXY_DATA_df))]
## Coerce columns to numeric data
OXY_DATA <- as.data.frame(lapply(OXY_DATA_df, function(x) as.numeric(x)))
## transpose data so metabolites are columns and samples are rows
OXY_DATA <- as.data.frame(t(OXY_DATA))
## reset metabolite names
colnames(OXY_DATA) <- rownames(OXY_DATA_df)
## Add label column with Sample IDs
OXY_DATA$label <- rownames(OXY_DATA)

## T2D Cutoff
T2DMETADATA <- read.csv("T2D_cutpoint_metabolomics.csv", header=TRUE, comment.char = "", check.names = FALSE)
## Join metabolomics sample metadata and experimental grouping data
SAMPLEMETADATA_T2D <- inner_join(OXY_SAMPLEMETADATA, T2DMETADATA, by="label")

## Extract sample IDs from sample meta data
oxyT2D_label <- SAMPLEMETADATA_T2D$label
## Extract sample meta data labels
oxymetanames <- colnames(SAMPLEMETADATA_T2D)
## Extract experimental data labels
oxyExperimentalGroups <- colnames(T2DMETADATA)

## Extract metabolomics samples that match sample IDs from sample meta data.
OXY_DATA_T2D <- OXY_DATA %>% filter(label %in% oxyT2D_label)

## Identify metabolites with data
colswithDAT <- sapply(OXY_DATA[,!(colnames(OXY_DATA) %in% "label")], function(x) sum(x %in% NA) != nrow(OXY_DATA))
## Remove metabolites containing all NAs
OXY_DATA_T2D_dat <- OXY_DATA[,colswithDAT]
## 20 metabolites with no data removed

## Function to determine % missing values
pcNA <- function(x) 100 * (sum(x %in% NA)/length(x))
## Calculate % missing values across each metabolite
OXY_DATA_T2D_pcNA <- OXY_DATA_T2D_dat %>% gather(Metabolites, value, -(label)) %>% 
	group_by(Metabolites) %>% summarise(pcNA = pcNA(value)) %>% arrange(desc(pcNA)) 
## Identify those with < 50% missing values
OXY_DATA_T2D_pcNA_keep <- OXY_DATA_T2D_pcNA %>% filter(pcNA < 50) %>% pull(Metabolites)
## Extract from data
OXY_DATA_T2D_nafilter <- OXY_DATA_T2D_dat[, colnames(OXY_DATA_T2D_dat) %in% c("label", OXY_DATA_T2D_pcNA_keep)]


## Check for univariate outliers	
## Run interative Grubbs test and get log transformed data back with outliers removed
OXY_DATA_T2D_naout <- OXY_DATA_T2D_nafilter %>% 
	gather(Metabolites, value, -(label)) %>%
	group_by(Metabolites) %>%
	mutate(value = grubbsOUT_FUN(value, "log", "0.01"))	
	
OXY_DATA_T2D_naoutCOMP <- OXY_DATA_T2D_naout

## % of data with NA
OXY_DATA_outRMpc <- 100 * (sum(OXY_DATA_T2D_naout$value %in% NA)/length(OXY_DATA_T2D_naout$value))
# > OXY_DATA_outRMpc
# [1] 5.163043

## % of original data with NAs
OXY_DATA_T2D_nafilter_melt <- OXY_DATA_T2D_nafilter %>% 
	gather(Metabolites, value, -(label)) 
NAsum_originalDAT <- OXY_DATA_T2D_nafilter_melt %>% 
	filter(value %in% NA) %>%
	nrow()
OXY_DATA_NApc <- 100 * (NAsum_originalDAT/nrow(OXY_DATA_T2D_nafilter_melt))
# > OXY_DATA_NApc
# [1] 4.939258

## How many outliers were removed
sumOutliers <- sum(OXY_DATA_T2D_naoutCOMP$value %in% NA) - NAsum_originalDAT
sumOutliers
# > sumOutliers
# [1] 14
## % of data removed from outlier removal
100 * (sumOutliers/length(OXY_DATA_T2D_naoutCOMP$value))
# > 100 * (sumOutliers/length(OXY_DATA_T2D_naoutCOMP$value)) 
# [1] 0.2237852
## ~ 0.2% of data removed by outlier test

## Per WCMC: 
## Sample 171-93-RED_1879 has noticeably high peaks on several of the analytes.  We have checked these and they 
## are all proper integrations by visual inspection, the Areas are several times the size of everything around it, and the, 
## internal standard “IS,” is not causing inflated values.

## Will check if RED_1879 is the cause of outliers.
OXY_DATA_T2D_naout_2 <- OXY_DATA_T2D_naout
OXY_DATA_T2D_naout_2$ID <- apply(OXY_DATA_T2D_naout_2[, colnames(OXY_DATA_T2D_naout_2) %in% c("label", "Metabolites")], 1, paste, collapse="_")
OXY_DATA_T2D_naout_2 <- OXY_DATA_T2D_naout_2 %>% mutate(NAlogic = value %in% NA)
OXY_DATA_T2D_nafilter_melt2 <- OXY_DATA_T2D_nafilter_melt
OXY_DATA_T2D_nafilter_melt2$ID <- apply(OXY_DATA_T2D_nafilter_melt2[, colnames(OXY_DATA_T2D_nafilter_melt2) %in% c("label", "Metabolites")], 1, paste, collapse="_")
OXY_DATA_T2D_nafilter_melt2 <- OXY_DATA_T2D_nafilter_melt2 %>% dplyr::mutate(rawNAvalue = value %in% NA) %>% dplyr::select(ID, rawNAvalue)
removedOutliersDAT <- inner_join(OXY_DATA_T2D_naout_2, OXY_DATA_T2D_nafilter_melt2) %>% 
	mutate(NAsum = NAlogic + rawNAvalue) %>% 
	filter(NAsum == 1) %>% 
	dplyr::select(label, Metabolites, value)
# > removedOutliersDAT
# A tibble: 14 x 3
# Groups:   Metabolites [13]
   # label    Metabolites      value
   # <chr>    <chr>            <dbl>
 # 1 RED_2083 6-keto-PGF1alpha    NA
 # 2 RED_1748 9,10-DiHOME         NA
 # 3 RED_1352 15,16-DiHODE        NA
 # 4 RED_1714 14,15-DiHETE        NA
 # 5 RED_1879 6-trans-LTB4        NA
 # 6 RED_1879 13S-HODE            NA
 # 7 RED_1879 9-HODE              NA
 # 8 RED_1879 15-HETE             NA
 # 9 RED_1879 5S-HEPE             NA
# 10 RED_1879 4-HDoHE             NA
# 11 RED_1879 5-KETE              NA
# 12 RED_1526 9,10-EpO            NA
# 13 RED_1484 9,10-EpO            NA
# 14 RED_1879 9(10)-EpOME         NA
sum(removedOutliersDAT$label %in% "RED_1879")
# > sum(removedOutliersDAT$label %in% "RED_1879") 
# [1] 8
## 8 of the 14 removed outliers were attributed to RED_1879.  

## Identify the % missing value per metabolite
OXY_DATA_T2D_metabNArate <- OXY_DATA_T2D_naout %>% 
	group_by(Metabolites) %>% 
	summarise(pcremoved = round(pcNA(value),1)) %>% 
	filter(pcremoved != 0) %>% 
	arrange(desc(pcremoved)) 
# A tibble: 24 x 2
   # Metabolites     pcremoved
   # <chr>               <dbl>
 # 1 19(20)-EpDPE         48.5
 # 2 17-HDoHE             36.8
 # 3 12(13)Ep-9-KODE      34.6
 # 4 15-KETE              30.1
 # 5 9,10-DiHODE          22.8
 # 6 10-Nitrooleate       16.2
 # 7 PGF2alpha             9.6
 # 8 6-trans-LTB4          8.8
 # 9 TXB2                  6.6
# 10 4-HDoHE               5.9
# ... with 14 more rows
## None > 50%

## Add sample metadata to filtered data
OXY_DATA_T2D_naoutWIDE <- as.data.frame(inner_join(SAMPLEMETADATA_T2D %>% dplyr::select(oxyExperimentalGroups), spread(OXY_DATA_T2D_naout, Metabolites, value), by="label"))
rownames(OXY_DATA_T2D_naoutWIDE) <- OXY_DATA_T2D_naoutWIDE$label
## Add sample metadata to original data
OXY_DATA_T2D_WIDE <- as.data.frame(inner_join(SAMPLEMETADATA_T2D %>% select(one_of(oxyExperimentalGroups)), OXY_DATA_T2D_dat, by="label"))
rownames(OXY_DATA_T2D_WIDE) <- OXY_DATA_T2D_WIDE$label

## Look at the distribution of sample data
par(mfrow=c(2,1))
boxplot(t(log(OXY_DATA_T2D_WIDE[,!(colnames(OXY_DATA_T2D_WIDE) %in% oxyExperimentalGroups)])), main = "Pre Outlier Removal", outline = FALSE,
	las = 3, whisklty = 0, staplelty = 0, col="lightblue")			
boxplot(t(OXY_DATA_T2D_naoutWIDE[,!(colnames(OXY_DATA_T2D_naoutWIDE) %in% oxyExperimentalGroups)]), main = "Post Outlier Removal", outline = FALSE,
	las = 3, whisklty = 0, staplelty = 0, col="pink")			
## RED_1879 sticks out pretty well.  Maybe RED_2325

## Order by experimental groups
OXY_DATA_T2D_WIDEorder <- OXY_DATA_T2D_WIDE[order(OXY_DATA_T2D_WIDE$GroupAbb, OXY_DATA_T2D_WIDE$label),]
OXY_DATA_T2D_naoutWIDEorder <- OXY_DATA_T2D_naoutWIDE[order(OXY_DATA_T2D_naoutWIDE$GroupAbb, OXY_DATA_T2D_naoutWIDE$label),]

par(mfrow=c(2,1))
boxplot(t(log(OXY_DATA_T2D_WIDEorder[,!(colnames(OXY_DATA_T2D_WIDEorder) %in% oxyExperimentalGroups)])), main = "Pre Outlier Removal", outline = FALSE,
	las = 3, whisklty = 0, staplelty = 0, col="lightblue", names=with(OXY_DATA_T2D_WIDEorder, paste(GroupAbb, label, sep="_")))			
boxplot(t(OXY_DATA_T2D_naoutWIDEorder[,!(colnames(OXY_DATA_T2D_naoutWIDEorder) %in% oxyExperimentalGroups)]), main = "Post Outlier Removal", outline = FALSE,
	las = 3, whisklty = 0, staplelty = 0, col="pink", names=with(OXY_DATA_T2D_naoutWIDEorder, paste(GroupAbb, label, sep="_")))			

## take exponential of filtered data to return to original values (data was logged transformed during outlier detection)
OXY_DATA_T2D_WIDEe <- OXY_DATA_T2D_naoutWIDE	
OXY_DATA_T2D_WIDEe[,!(colnames(OXY_DATA_T2D_WIDEe) %in% oxyExperimentalGroups)] <- 
	exp(OXY_DATA_T2D_naoutWIDE[,!(colnames(OXY_DATA_T2D_naoutWIDE) %in% oxyExperimentalGroups)])

## Knn Imputation for multivariate analyses.
OXY_DATA_T2D_DF <- OXY_DATA_T2D_WIDEe
knnOXY_DATA_T2D <- impute.knn(as.matrix(OXY_DATA_T2D_WIDEe[,!(colnames(OXY_DATA_T2D_WIDEe) %in% oxyExperimentalGroups)]))
OXY_DATA_T2D_DF[,!(colnames(OXY_DATA_T2D_DF) %in% oxyExperimentalGroups)] <- data.frame(knnOXY_DATA_T2D$data)	

## Check distributions with imputed data
par(mfrow=c(3,1))
boxplot(t(log(OXY_DATA_T2D_WIDE[,!(colnames(OXY_DATA_T2D_WIDE) %in% oxyExperimentalGroups)])), main = "Pre Outlier Removal", outline = FALSE,
	las = 3, whisklty = 0, staplelty = 0, col="lightblue")			
boxplot(t(OXY_DATA_T2D_naoutWIDE[,!(colnames(OXY_DATA_T2D_naoutWIDE) %in% oxyExperimentalGroups)]), main = "Post Outlier Removal", outline = FALSE,
	las = 3, whisklty = 0, staplelty = 0, col="pink")			
boxplot(t(log(OXY_DATA_T2D_DF[,!(colnames(OXY_DATA_T2D_DF) %in% oxyExperimentalGroups)])), main = "Post Outlier Imputation", outline = FALSE,
	las = 3, whisklty = 0, staplelty = 0, col="lightgreen")	

## PCA
OXY_DATA_T2Dpca <- prcomp(scale(log(OXY_DATA_T2D_DF[,!(colnames(OXY_DATA_T2D_DF) %in% oxyExperimentalGroups)])), center=FALSE)
## Extract scores and join with metadata
OXY_DATA_T2Dpca_SCORES <- as.data.frame(OXY_DATA_T2Dpca$x)
OXY_DATA_T2Dpca_SCORES$label <- rownames(OXY_DATA_T2Dpca_SCORES)
OXY_DATA_T2Dpca_SCORES <- inner_join(OXY_DATA_T2D_DF[,colnames(OXY_DATA_T2D_DF) %in% oxyExperimentalGroups], OXY_DATA_T2Dpca_SCORES, by="label")

## Plot Scores and loadings
par(mfrow=c(2,2))
ellipseplot(OXY_DATA_T2Dpca_SCORES$PC1, OXY_DATA_T2Dpca_SCORES$PC2,OXY_DATA_T2Dpca_SCORES$Disease,   
	pcol=c("#e41a1c","#377eb8"),pbgcol=FALSE, legpos="topleft",cexsize=2, ppch=c(21:22), legcexsize=1.5)
title(xlab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC1"]], ylab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC2"]], main="PCA - overlayed with Alzheimer's status")
ellipseplot(OXY_DATA_T2Dpca_SCORES$PC1, OXY_DATA_T2Dpca_SCORES$PC2,OXY_DATA_T2Dpca_SCORES$Metab,   
	pcol=c("#4daf4a","#984ea3"),pbgcol=FALSE, legpos="topleft",cexsize=2, ppch=c(21:22), legcexsize=1.5)
title(xlab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC1"]], ylab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC2"]], main="PCA - overlayed with Diabetes status")
ellipseplot(OXY_DATA_T2Dpca_SCORES$PC1, OXY_DATA_T2Dpca_SCORES$PC2,OXY_DATA_T2Dpca_SCORES$GroupAbb,   
	pcol=c("#e41a1c","#377eb8","#4daf4a","#984ea3"),pbgcol=FALSE, legpos="topleft",
	cexsize=2, ppch=c(21:24), legcexsize=1.5)
title(xlab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC1"]], ylab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC2"]], main="PCA - overlayed with Alzheimer's and diabetes status")
plot(OXY_DATA_T2Dpca_SCORES$PC1, OXY_DATA_T2Dpca_SCORES$PC2, type="n", main="PCA - overlayed with sample IDs")
text(OXY_DATA_T2Dpca_SCORES$PC1, OXY_DATA_T2Dpca_SCORES$PC2, labels=OXY_DATA_T2Dpca_SCORES$label)

layout(matrix(c(1,1,2,2,1,1,2,2,1,1,3,3,1,1,3,3), nrow=4))	
boxplot(t(log(OXY_DATA_T2D_DF[,!(colnames(OXY_DATA_T2D_DF) %in% oxyExperimentalGroups)])), main = "Post Outlier Imputation", outline = FALSE,
	las = 3, whisklty = 0, staplelty = 0, col="lightgreen")	
ellipseplot(OXY_DATA_T2Dpca_SCORES$PC1, OXY_DATA_T2Dpca_SCORES$PC2,OXY_DATA_T2Dpca_SCORES$GroupAbb,   
	pcol=c("#e41a1c","#377eb8","#4daf4a","#984ea3"),pbgcol=FALSE, legpos="topleft",
	cexsize=2, ppch=c(21:24), legcexsize=1.5)
title(xlab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC1"]], ylab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC2"]], main="PCA - overlayed with Alzheimer's and diabetes status")
plot(OXY_DATA_T2Dpca_SCORES$PC1, OXY_DATA_T2Dpca_SCORES$PC2, type="n")
text(OXY_DATA_T2Dpca_SCORES$PC1, OXY_DATA_T2Dpca_SCORES$PC2, labels=OXY_DATA_T2Dpca_SCORES$label)

## Consider removing RED_1525, _1834, _0870, _2325, _1879

## will remove RED_1879 from outlier removed data
## And then redo exponential transformation and imputations
OXY_DATA_T2D_WIDEpp_rmsamp <- droplevels(as.data.frame(OXY_DATA_T2D_naoutWIDE[!(rownames(OXY_DATA_T2D_naoutWIDE) %in% "RED_1879"),]))
OXY_DATA_T2D_WIDEe_rmsamp <- OXY_DATA_T2D_WIDEpp_rmsamp
OXY_DATA_T2D_WIDEe_rmsamp[,!(colnames(OXY_DATA_T2D_WIDEe_rmsamp) %in% oxyExperimentalGroups)] <- 
	exp(OXY_DATA_T2D_WIDEpp_rmsamp[,!(colnames(OXY_DATA_T2D_WIDEpp_rmsamp) %in% oxyExperimentalGroups)])
	
## Knn Imputation for multivariate analyses.
OXY_DATA_T2D_DF_rmsamp <- OXY_DATA_T2D_WIDEe_rmsamp
knnOXY_DATA_T2D_rmsamp <- impute.knn(as.matrix(OXY_DATA_T2D_WIDEe_rmsamp[,!(colnames(OXY_DATA_T2D_WIDEe_rmsamp) %in% oxyExperimentalGroups)]))	
OXY_DATA_T2D_DF_rmsamp[,!(colnames(OXY_DATA_T2D_DF_rmsamp) %in% oxyExperimentalGroups)] <- data.frame(knnOXY_DATA_T2D_rmsamp$data)	

## PCA with 1879 removed
OXY_DATA_T2Dpca_rmsamp <- prcomp(scale(log(OXY_DATA_T2D_DF_rmsamp[,!(colnames(OXY_DATA_T2D_DF_rmsamp) %in% oxyExperimentalGroups)])), center=FALSE)

## Extract scores and join with metadata
OXY_DATA_T2Dpca_rmsampSCORES <- as.data.frame(OXY_DATA_T2Dpca_rmsamp$x)
OXY_DATA_T2Dpca_rmsampSCORES$label <- rownames(OXY_DATA_T2Dpca_rmsampSCORES)
OXY_DATA_T2Dpca_rmsampSCORES <- inner_join(OXY_DATA_T2D_DF[,colnames(OXY_DATA_T2D_DF) %in% oxyExperimentalGroups], OXY_DATA_T2Dpca_rmsampSCORES, by="label")

## Plot Scores and loadings
par(mfrow=c(2,2))
ellipseplot(OXY_DATA_T2Dpca_rmsampSCORES$PC1, OXY_DATA_T2Dpca_rmsampSCORES$PC2,OXY_DATA_T2Dpca_rmsampSCORES$Disease,   
	pcol=c("#e41a1c","#377eb8"),pbgcol=FALSE, legpos="topleft",cexsize=2, ppch=c(21:22), legcexsize=1.5)
title(xlab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC1"]], ylab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC2"]], main="PCA - overlayed with Alzheimer's status")
ellipseplot(OXY_DATA_T2Dpca_rmsampSCORES$PC1, OXY_DATA_T2Dpca_rmsampSCORES$PC2,OXY_DATA_T2Dpca_rmsampSCORES$Metab,   
	pcol=c("#4daf4a","#984ea3"),pbgcol=FALSE, legpos="topleft",cexsize=2, ppch=c(21:22), legcexsize=1.5)
title(xlab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC1"]], ylab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC2"]], main="PCA - overlayed with Diabetes status")
ellipseplot(OXY_DATA_T2Dpca_rmsampSCORES$PC1, OXY_DATA_T2Dpca_rmsampSCORES$PC2,OXY_DATA_T2Dpca_rmsampSCORES$GroupAbb,   
	pcol=c("#e41a1c","#377eb8","#4daf4a","#984ea3"),pbgcol=FALSE, legpos="topleft",
	cexsize=2, ppch=c(21:24), legcexsize=1.5)
title(xlab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC1"]], ylab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC2"]], main="PCA - overlayed with Alzheimer's and diabetes status")
plot(OXY_DATA_T2Dpca_rmsampSCORES$PC1, OXY_DATA_T2Dpca_rmsampSCORES$PC2, type="n", main="PCA - overlayed with sample IDs")
text(OXY_DATA_T2Dpca_rmsampSCORES$PC1, OXY_DATA_T2Dpca_rmsampSCORES$PC2, labels=OXY_DATA_T2Dpca_rmsampSCORES$label)

layout(matrix(c(1,1,2,2,1,1,2,2,1,1,3,3,1,1,3,3), nrow=4))	
boxplot(t(log(OXY_DATA_T2D_DF[,!(colnames(OXY_DATA_T2D_DF) %in% oxyExperimentalGroups)])), main = "Post Outlier Imputation", outline = FALSE,
	las = 3, whisklty = 0, staplelty = 0, col="lightgreen")	
ellipseplot(OXY_DATA_T2Dpca_rmsampSCORES$PC1, OXY_DATA_T2Dpca_rmsampSCORES$PC2,OXY_DATA_T2Dpca_rmsampSCORES$GroupAbb,   
	pcol=c("#e41a1c","#377eb8","#4daf4a","#984ea3"),pbgcol=FALSE, legpos="topleft",
	cexsize=2, ppch=c(21:24), legcexsize=1.5)
title(xlab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC1"]], ylab=PCAvarAxis(OXY_DATA_T2Dpca)[["PC2"]], main="PCA - overlayed with Alzheimer's and diabetes status")
plot(OXY_DATA_T2Dpca_rmsampSCORES$PC1, OXY_DATA_T2Dpca_rmsampSCORES$PC2, type="n")
text(OXY_DATA_T2Dpca_rmsampSCORES$PC1, OXY_DATA_T2Dpca_rmsampSCORES$PC2, labels=OXY_DATA_T2Dpca_rmsampSCORES$label)

## Consider removing RED_1525, _1834, _0870, _2325

	
#===============================================================================================
#
# Group Comparisons
#
#===============================================================================================

## Make new data frame to collect summed subclasses of metabolites 
data.frame(orig=OXY_DATA_T2D_DF[!(grepl("RED_1879", rownames(OXY_DATA_T2D_DF))),"9,10-DiHODE"], exp=OXY_DATA_T2D_WIDEe_rmsamp[,"9,10-DiHODE"])
OXY_DATA_T2D_DF_GC <- OXY_DATA_T2D_WIDEe_rmsamp
## Filter metabolite meta data to match filtered data.
OXY_METABMETADATA_GC <- droplevels(OXY_METABMETADATA[OXY_METABMETADATA[,grepl("approved", colnames(OXY_METABMETADATA))] %in% 
	colnames(OXY_DATA_T2D_DF_GC)[!(colnames(OXY_DATA_T2D_DF_GC) %in% oxyExperimentalGroups)],])
## Make row names of metabolite meta data the metabolites
rownames(OXY_METABMETADATA_GC) <- OXY_METABMETADATA_GC[,grepl("approved", colnames(OXY_METABMETADATA_GC))]

## Sum subclasses and add columns to data
for(i in unique(OXY_METABMETADATA_GC$class)){
	oxys <- rownames(OXY_METABMETADATA_GC)[OXY_METABMETADATA_GC$class %in% i]
	OXY_DATA_T2D_DF_GC[[paste("sum",  i, sep="_")]] <- rowSums(OXY_DATA_T2D_DF_GC[,colnames(OXY_DATA_T2D_DF_GC) %in% oxys, drop=FALSE])
}

## Remove unknown precursor
FAprecursors <- levels(OXY_METABMETADATA_GC$Faprecursor)
FAprecursors <- FAprecursors[!(FAprecursors %in% "")]
## Sum FA precursors and add columns to data
for(i in FAprecursors){
	oxys <- rownames(OXY_METABMETADATA_GC)[OXY_METABMETADATA_GC$Faprecursor %in% i]
	OXY_DATA_T2D_DF_GC[[paste("sum", "prec", i, sep="_")]] <- rowSums(OXY_DATA_T2D_DF_GC[,colnames(OXY_DATA_T2D_DF_GC) %in% oxys, drop=FALSE])
}


## Remove single class and precursor data columns 
singleCLASS <- names(summary(factor(OXY_METABMETADATA_GC$class)))[summary(factor(OXY_METABMETADATA_GC$class)) == 1]
singleCLASScolnames <- sapply(singleCLASS, function(x) colnames(OXY_DATA_T2D_DF_GC)[grepl(x, colnames(OXY_DATA_T2D_DF_GC))])
singlePRECUR <- names(summary(factor(OXY_METABMETADATA_GC$Faprecursor)))[summary(factor(OXY_METABMETADATA_GC$Faprecursor)) == 1]
singlePRECUR <- singlePRECUR[!(singlePRECUR %in% "")]
singlePRECURcolnames <- sapply(singlePRECUR, function(x) colnames(OXY_DATA_T2D_DF_GC)[grepl(x, colnames(OXY_DATA_T2D_DF_GC))])

OXY_DATA_T2D_DF_GC <- OXY_DATA_T2D_DF_GC[, !(colnames(OXY_DATA_T2D_DF_GC) %in% c(singleCLASScolnames,singlePRECURcolnames))]




KWmedFUN <- function(levs, meltDAT, fac) {
	dat <- meltDAT[meltDAT$Metabolites %in% levs,]
	out <- dat %>% group_by_(fac)%>% 
		summarise(MEDIAN = signif(median(value, na.rm=TRUE), 4), Q25=signif(quantile(value, na.rm=TRUE)["25%"], 4), Q75=signif(quantile(value, na.rm=TRUE)["75%"], 4)) %>% 
		as.data.frame() %>%
		gather(STAT, value, -(fac))
	
	MEDQ <- out[,"value"]
	names(MEDQ) <- apply(out[,1:2], 1, paste, collapse="_")
	flevs <- list()
	for(i in levels(factor(dat[,fac]))){
		flevs[[i]] <- grep(i, names(MEDQ))
	}
	if(nlevels(factor(dat[,fac])) > 2) {
		output <- c(MEDQ[unlist(flevs)], round(kruskal.test(formula(paste("value~", fac)), data=dat)$p.value,5))
	} else {
		output <- c(MEDQ[unlist(flevs)], round(wilcox.test(formula(paste("value~", fac)), data=dat)$p.value,5))
	}
	names(output)[length(output)] <- "KW_P"
	output
}
# lapply(unique(OXY_DATA_T2D_DF_GC_TALL_MH$Metabolites), KWmedFUN, OXY_DATA_T2D_DF_GC_TALL_MH, "Disease")
# OXY_DATA_T2D_DF_GC_TALL_MH[OXY_DATA_T2D_DF_GC_TALL_MH$Metabolites %in% unique(OXY_DATA_T2D_DF_GC_TALL_MH$Metabolites)[1],]
# OXY_DATA_T2D_DF_GC_TALL[OXY_DATA_T2D_DF_GC_TALL$Metabolites %in% unique(OXY_DATA_T2D_DF_GC_TALL$Metabolites)[1],]
## Raw data
OXY_DATA_T2D_DF_GC_TALL <- OXY_DATA_T2D_DF_GC %>% gather(Metabolites, value, -(label:APOEe4_carrier))
OXY_DATA_T2D_DF_ALLKW <- data.frame(t(sapply(unique(OXY_DATA_T2D_DF_GC_TALL$Metabolites), KWmedFUN, OXY_DATA_T2D_DF_GC_TALL, "GroupAbb")))
OXY_DATA_T2D_DF_ALLKW$KW_FDR <- p.adjust(OXY_DATA_T2D_DF_ALLKW$KW_P, "fdr")

## AD diff in MH
OXY_DATA_T2D_DF_GC_TALL_MH <- droplevels(OXY_DATA_T2D_DF_GC_TALL[OXY_DATA_T2D_DF_GC_TALL$Metab %in% "MH",])
OXY_DATA_T2D_DF_MHKW <- data.frame(t(sapply(unique(OXY_DATA_T2D_DF_GC_TALL_MH$Metabolites), KWmedFUN, OXY_DATA_T2D_DF_GC_TALL_MH, "Disease")))
OXY_DATA_T2D_DF_MHKW$KW_FDR <- p.adjust(OXY_DATA_T2D_DF_MHKW$KW_P, "fdr")
OXY_DATA_T2D_DF_MHKW$Metabolites <- rownames(OXY_DATA_T2D_DF_MHKW)

## Subset columns from metabolite meta data
OXY_METABMETADATA_GC_joinDF <- OXY_METABMETADATA_GC %>% 
	mutate(Metabolites=approved.short.names) %>% 
	dplyr::select(Metabolites, class, Faprecursor) 

## Create Table 1, with added columns
OXYTable1 <- OXY_DATA_T2D_DF_MHKW %>% 
	mutate(
		AD = paste0(signif(AD_MEDIAN,3), " (", signif(AD_Q25,2), ", ", signif(AD_Q75,2),")"),
		ND = paste0(signif(ND_MEDIAN,3), " (", signif(ND_Q25,2), ", ", signif(ND_Q75,2),")")
	) %>%
	dplyr::select(Metabolites, ND, AD, KW_P, KW_FDR) %>%
	mutate(KW_P = round(KW_P, 3), KW_FDR = round(KW_FDR, 3)) %>% 
	full_join(OXY_METABMETADATA_GC_joinDF, by="Metabolites") %>%
	mutate(
		Faprecursor = as.character(Faprecursor),
		Metabolites = gsub("sum_", "", Metabolites),
		Metabolites = gsub("prec_", "", Metabolites)
	)
	
## Make function to capitalize first letter in words.
simpleCap <- function(x) {
  s <- strsplit(x, " ")[[1]]
  paste(toupper(substring(s, 1,1)), substring(s, 2),
      sep="", collapse=" ")
}

## Capitalize first word in metabolites for summed species
OXYTable1[OXYTable1$Metabolites %in% unique(OXYTable1$class),"Metabolites"] <- paste("Total", 
	sapply(OXYTable1[OXYTable1$Metabolites %in% unique(OXYTable1$class),"Metabolites"], simpleCap)
)
OXYTable1$class <- sapply(OXYTable1$class, simpleCap)
## Add Sub-class to summed sub classes
OXYTable1[OXYTable1$Metabolites %in% paste("Total", unique(OXYTable1$class)),"class"] <- 
	gsub("Total ", "", OXYTable1[OXYTable1$Metabolites %in% paste("Total", unique(OXYTable1$class)),"Metabolites"])
## Change remaining NANA to NA
OXYTable1[OXYTable1$class %in% "NANA","class"] <- NA

## Change summed FA precursors '_' to ':'
OXYTable1[OXYTable1$Metabolites %in% unique(OXYTable1$Faprecursor),"Metabolites"] <- 
	gsub("_", ":", OXYTable1[OXYTable1$Metabolites %in% unique(OXYTable1$Faprecursor),"Metabolites"])
## Change FA precursors '_' to ':'
OXYTable1$Faprecursor <- gsub("_", ":", OXYTable1$Faprecursor)
## Add FA precursur to summed FA precursor
OXYTable1[OXYTable1$Metabolites %in% unique(OXYTable1$Faprecursor),"Faprecursor"] <- 
	OXYTable1[OXYTable1$Metabolites %in% unique(OXYTable1$Faprecursor),"Metabolites"]
	
## Arrange table so it is ordered by class then P value
OXYTable1 <- OXYTable1 %>% arrange(class, KW_P)

## Identify n for groups
MH_ADn <- summary(OXY_DATA_T2D_DF_GC$GroupAbb)[names(summary(OXY_DATA_T2D_DF_GC$GroupAbb)) == "MH_AD"]
MH_NDn <- summary(OXY_DATA_T2D_DF_GC$GroupAbb)[names(summary(OXY_DATA_T2D_DF_GC$GroupAbb)) == "MH_ND"]

## Create flextable object
OxyTable1_ft <- flextable(data = OXYTable1) %>% 
	set_header_labels( 
		ND = paste0("Cognitive Healthy (n=", MH_NDn, ")"), 
		AD = paste0("Alzheirmer's Disease (n=", MH_NDn, ")"), 
		KW_P = "P",
		KW_FDR = "FDR",
		class = "Sub-Class",
		Faprecursor = "Fatty Acid Precursor"
	) %>% 
	# bg(i = ~ KW_P <= 0.05, bg = "#b2df8a") %>% 
	italic(j = c(4,5), italic = TRUE, part="header") %>% 
	bold(i = ~ KW_P <= 0.05, bold = TRUE) %>% 
	font(font="Arial")  %>%
	## Footnotes is pushing landscape out of margins. 
	# footnote(j = 1:5,
            # value = as_paragraph(
              # c("Assessed by UPLC-MS/MS. Units are nM/mL.",
                # "Values are median (1st quartile, 3rd quartile).",
                # "Values are median (1st quartile, 3rd quartile).",
                # "Derived from Mann Whitney U test.",
                # "Derived from Benjamini and Hochberg FDR correction")
            # ),
            # ref_symbols = c("1", "2", "2", "3", "4"),
            # part = "header") %>%
	autofit(add_h=0) 		

## Will save docx file, but Word doc will need to be open, changed to landscape, and unselect 'repeate header rows'
# read_docx() %>% 
  # body_add_flextable(OxyTable1_ft) %>% 
  # print(target = "./Tables/Oxy AD comp MH 7-13-19.docx")
  
  
## AD diff in T2D
OXY_DATA_T2D_DFmelt_T2D <- droplevels(OXY_DATA_T2D_DF_GC_TALL[OXY_DATA_T2D_DF_GC_TALL$Metab %in% "T2D",])
OXY_DATA_T2D_DF_T2DKW <- data.frame(t(sapply(unique(OXY_DATA_T2D_DFmelt_T2D$Metabolites), KWmedFUN, OXY_DATA_T2D_DFmelt_T2D, "Disease")))
OXY_DATA_T2D_DF_T2DKW$KW_FDR <- p.adjust(OXY_DATA_T2D_DF_T2DKW$KW_P, "fdr")
OXY_DATA_T2D_DF_T2DKW$Metabolites <- rownames(OXY_DATA_T2D_DF_T2DKW)


## Create Table 2, with added columns
OXYTable2 <- OXY_DATA_T2D_DF_T2DKW %>% 
	mutate(
		AD = paste0(signif(AD_MEDIAN,3), " (", signif(AD_Q25,2), ", ", signif(AD_Q75,2),")"),
		ND = paste0(signif(ND_MEDIAN,3), " (", signif(ND_Q25,2), ", ", signif(ND_Q75,2),")")
	) %>%
	dplyr::select(Metabolites, ND, AD, KW_P, KW_FDR) %>%
	mutate(KW_P = round(KW_P, 3), KW_FDR = round(KW_FDR, 3)) %>% 
	full_join(OXY_METABMETADATA_GC_joinDF, by="Metabolites") %>%
	mutate(
		Faprecursor = as.character(Faprecursor),
		Metabolites = gsub("sum_", "", Metabolites),
		Metabolites = gsub("prec_", "", Metabolites)
	)

## Capitalize first word in metabolites for summed species
OXYTable2[OXYTable2$Metabolites %in% unique(OXYTable2$class),"Metabolites"] <- paste("Total", 
	sapply(OXYTable2[OXYTable2$Metabolites %in% unique(OXYTable2$class),"Metabolites"], simpleCap)
)
OXYTable2$class <- sapply(OXYTable2$class, simpleCap)
## Add Sub-class to summed sub classes
OXYTable2[OXYTable2$Metabolites %in% paste("Total", unique(OXYTable2$class)),"class"] <- 
	gsub("Total ", "", OXYTable2[OXYTable2$Metabolites %in% paste("Total", unique(OXYTable2$class)),"Metabolites"])
## Change remaining NANA to NA
OXYTable2[OXYTable2$class %in% "NANA","class"] <- NA

## Change summed FA precursors '_' to ':'
OXYTable2[OXYTable2$Metabolites %in% unique(OXYTable2$Faprecursor),"Metabolites"] <- 
	gsub("_", ":", OXYTable2[OXYTable2$Metabolites %in% unique(OXYTable2$Faprecursor),"Metabolites"])
## Change FA precursors '_' to ':'
OXYTable2$Faprecursor <- gsub("_", ":", OXYTable2$Faprecursor)
## Add FA precursur to summed FA precursor
OXYTable2[OXYTable2$Metabolites %in% unique(OXYTable2$Faprecursor),"Faprecursor"] <- 
	OXYTable2[OXYTable2$Metabolites %in% unique(OXYTable2$Faprecursor),"Metabolites"]
	
## Arrange table so it is ordered by class then P value
OXYTable2 <- OXYTable2 %>% arrange(class, KW_P)

## Identify n for groups
T2D_ADn <- summary(OXY_DATA_T2D_DF_GC$GroupAbb)[names(summary(OXY_DATA_T2D_DF_GC$GroupAbb)) == "T2D_AD"]
T2D_NDn <- summary(OXY_DATA_T2D_DF_GC$GroupAbb)[names(summary(OXY_DATA_T2D_DF_GC$GroupAbb)) == "T2D_ND"]

## Create flextable object
OxyTable2_ft <- flextable(data = OXYTable2) %>% 
	set_header_labels( 
		ND = paste0("Cognitive Healthy (n=", T2D_ADn, ")"), 
		AD = paste0("Alzheirmer's Disease (n=", T2D_NDn, ")"), 
		KW_P = "P",
		KW_FDR = "FDR",
		class = "Sub-Class",
		Faprecursor = "Fatty Acid Precursor"
	) %>% 
	# bg(i = ~ KW_P <= 0.05, bg = "#b2df8a") %>% 
	italic(j = c(4,5), italic = TRUE, part="header") %>% 
	bold(i = ~ KW_P <= 0.05, bold = TRUE) %>% 
	font(font="Arial")  %>%
	## Footnotes is pushing landscape out of margins. 
	# footnote(j = 1:5,
            # value = as_paragraph(
              # c("Assessed by UPLC-MS/MS. Units are nM/mL.",
                # "Values are median (1st quartile, 3rd quartile).",
                # "Values are median (1st quartile, 3rd quartile).",
                # "Derived from Mann Whitney U test.",
                # "Derived from Benjamini and Hochberg FDR correction")
            # ),
            # ref_symbols = c("1", "2", "2", "3", "4"),
            # part = "header") %>%
	autofit(add_h=0) 		

## Will save docx file, but Word doc will need to be open, changed to landscape, and unselect 'repeate header rows'
# read_docx() %>% 
  # body_add_flextable(OxyTable2_ft) %>% 
  # print(target = "./Tables/Oxy AD comp T2D 7-14-19.docx")


## T2D diff in ND
OXY_DATA_T2D_DFmelt_ND <- droplevels(OXY_DATA_T2D_DF_GC_TALL[OXY_DATA_T2D_DF_GC_TALL$Disease %in% "ND",])
OXY_DATA_T2D_DF_NDKW <- data.frame(t(sapply(unique(OXY_DATA_T2D_DFmelt_ND$Metabolites), KWmedFUN, OXY_DATA_T2D_DFmelt_ND, "Metab")))
OXY_DATA_T2D_DF_NDKW$KW_FDR <- p.adjust(OXY_DATA_T2D_DF_NDKW$KW_P, "fdr")


## T2D diff in AD
OXY_DATA_T2D_DFmelt_AD <- droplevels(OXY_DATA_T2D_DF_GC_TALL[OXY_DATA_T2D_DF_GC_TALL$Disease %in% "AD",])
OXY_DATA_T2D_DF_ADKW <- data.frame(t(sapply(unique(OXY_DATA_T2D_DFmelt_AD$Metabolites), KWmedFUN, OXY_DATA_T2D_DFmelt_AD, "Metab")))
OXY_DATA_T2D_DF_ADKW$KW_FDR <- p.adjust(OXY_DATA_T2D_DF_ADKW$KW_P, "fdr")


#===============================================================================================
#
# Modeling Exercise
#
#===============================================================================================

##############################################
## PLSDA in subject with diabetes
##############################################

OXY_T2D <- OXY_DATA_T2D_DF[OXY_DATA_T2D_DF$Metab == "T2D",]
OXY_T2D_X <- OXY_T2D[,!(colnames(OXY_T2D) %in% oxyExperimentalGroups)]
OXY_T2D_dis <- factor(OXY_T2D$Disease)

## Caret PLSDA
## Set training function for PLSDA
trainControl <- trainControl(
	method = "repeatedcv", # training method
	number=10, # number of folds
	repeats = 10, # number of complete sets of folds
	classProbs = TRUE, # class probabilities
	summaryFunction = defaultSummary)

## Fit model
set.seed(1000) # set seed to reproduce results
OXY_T2D_PLSDAcaret <- train(scale(log(round(OXY_T2D_X,2))), # Exploratory variables
	OXY_T2D_dis, # Dependent variable (classifications)
	method = "pls", # algorithm
	tuneLength = 10, # number of latent variables
	trControl = trainControl, # control parameters
	preProc = NULL
)
OXY_T2D_PLSDAcaret
# > OXY_T2D_PLSDAcaret
# Partial Least Squares 

# 42 samples
# 46 predictors
 # 2 classes: 'AD', 'ND' 

# No pre-processing
# Resampling: Cross-Validated (10 fold, repeated 10 times) 
# Summary of sample sizes: 38, 37, 38, 37, 38, 38, ... 
# Resampling results across tuning parameters:

  # ncomp  Accuracy   Kappa     
   # 1     0.5430000  0.09234199
   # 2     0.6170000  0.24029371
   # 3     0.5650000  0.13723543
   # 4     0.5900000  0.17757343
   # 5     0.5811667  0.16801998
   # 6     0.5710000  0.14882917
   # 7     0.5831667  0.17302764
   # 8     0.5963333  0.19928072
   # 9     0.6236667  0.25089744
  # 10     0.6193333  0.24345788

# Accuracy was used to select the optimal model using the largest value.
# The final value used for the model was ncomp = 9.

## Pull out resampling data
OXY_T2D_PLSDAcaret_CVres <- OXY_T2D_PLSDAcaret$resample
## Make Fold and Repeat column
OXY_T2D_PLSDAcaret_CVres$Fold <- sapply(strsplit(OXY_T2D_PLSDAcaret_CVres$Resample, "\\."), function(x) x[1])
OXY_T2D_PLSDAcaret_CVres$Rep <- sapply(strsplit(OXY_T2D_PLSDAcaret_CVres$Resample, "\\."), function(x) x[2])
## order by Repeat and then folds
OXY_T2D_PLSDAcaret_CVres <- OXY_T2D_PLSDAcaret_CVres[order(OXY_T2D_PLSDAcaret_CVres$Rep, OXY_T2D_PLSDAcaret_CVres$Fold),]

## Mean Accuracy aross fold - should equal LV with best CV results
mean(OXY_T2D_PLSDAcaret_CVres$Accuracy)

## Calculate mean, and sd of Repeats adn then also attach overall mean and sd.
OXY_T2D_PLSDAcaret_CVres_MEAN <- OXY_T2D_PLSDAcaret_CVres %>% group_by(Rep) %>% 
	summarise(MEAN=mean(100*Accuracy), SD=sd(100*Accuracy)) %>%
	bind_rows(data.frame(Rep="All", MEAN=mean(100*OXY_T2D_PLSDAcaret_CVres$Accuracy), SD=sd(100*OXY_T2D_PLSDAcaret_CVres$Accuracy))) %>%
	as.data.frame()
## Set row names as Repeat labels
rownames(OXY_T2D_PLSDAcaret_CVres_MEAN) <- OXY_T2D_PLSDAcaret_CVres_MEAN$Rep
## Remove Rep column
OXY_T2D_PLSDAcaret_CVres_MEAN$Rep <- NULL
## Calculate upper and lower mean+sd
OXY_T2D_PLSDAcaret_CVres_MEAN$Upper <- with(OXY_T2D_PLSDAcaret_CVres_MEAN, MEAN+SD)
OXY_T2D_PLSDAcaret_CVres_MEAN$Lower <- with(OXY_T2D_PLSDAcaret_CVres_MEAN, MEAN-SD)

## Plot mean CV results across each LV and mean CV across repeats of best LV
# pdf("./Figures/T2D PLSDA Caret CV results 71519.pdf", height=12, width=7, useDingbats=FALSE)	
par(mfrow=c(2,1))
plot(OXY_T2D_PLSDAcaret$results$ncomp, 100*OXY_T2D_PLSDAcaret$results$Accuracy, type="l", pch=21, bg="black", ylim=c(0,100),
	ylab="Average Cross-Validation Accuracy, %", xlab="Latent Variable")
OXY_T2D_PLSDAcaret_CVres_MEAN_bp <- barplot(as.matrix(OXY_T2D_PLSDAcaret_CVres_MEAN$MEAN), beside=TRUE, ylim=c(0, max(as.matrix(OXY_T2D_PLSDAcaret_CVres_MEAN))),
	names.arg=rownames(OXY_T2D_PLSDAcaret_CVres_MEAN), col=c(rep("#FFFFFF", 10), "gray"), ylab="Average Cross-Validation Accuracy, %")
segments(OXY_T2D_PLSDAcaret_CVres_MEAN_bp, OXY_T2D_PLSDAcaret_CVres_MEAN$Upper, OXY_T2D_PLSDAcaret_CVres_MEAN_bp, OXY_T2D_PLSDAcaret_CVres_MEAN$Lower)
# dev.off()

## Identify LV with best CV results
OXY_T2D_PLSDAcaret_bestTune <- unlist(OXY_T2D_PLSDAcaret$bestTune)
## Create data frame for pls function
OXY_T2D_plsdaDF <- data.frame(Y=as.integer(OXY_T2D_dis), X=I(scale(log(OXY_T2D_X))))
## PLSDA with tuned LV
OXY_T2Dplsda <- pls::mvr(Y~., data=OXY_T2D_plsdaDF, ncomp=OXY_T2D_PLSDAcaret_bestTune, method="oscorespls", validation="CV") 
## Boostrapped VIP 
OXY_T2D_plsdaVIPboot <- plsdaVIPboot_FUN(OXY_T2D_plsdaDF, nc=OXY_T2D_PLSDAcaret_bestTune, iter=500, Ytype="Categorical")

## Use Bootstrapped BCA CI
OXY_T2D_plsdaVIPbootbca <- OXY_T2D_plsdaVIPboot$BOOTCI[["bca"]]
## Make Metabolites column
OXY_T2D_plsdaVIPbootbca$Metabolites <- rownames(OXY_T2D_plsdaVIPbootbca)
## Make data frame with Metabolites and VIP calculations
OXY_T2Dplsda_VIPDF <- data.frame(Metabolites=names(colMeans(VIP(OXY_T2Dplsda)[1:OXY_T2D_PLSDAcaret_bestTune,])), 
	VIP=colMeans(VIP(OXY_T2Dplsda)[1:OXY_T2D_PLSDAcaret_bestTune,]))
## Join VIPs with boostrapped VIP CIs
OXY_T2D_plsdaVIPbootbca <- full_join(OXY_T2Dplsda_VIPDF, OXY_T2D_plsdaVIPbootbca, by="Metabolites") %>%
	arrange(VIP) %>%
	mutate(Color = ifelse(VIP > 1, "black", "grey"), )

## How many VIP > 1
sum(OXY_T2D_plsdaVIPbootbca$VIP > 1)
# > sum(OXY_T2D_plsdaVIPbootbca$VIP > 1) 
# [1] 19

## Plot VIP and bootstrapped VIP CIs
# pdf("./Figures/T2D PLSDA VIP Plot 71519.pdf", height=10, width=6, useDingbats=FALSE)	
par(mfrow=c(1,1))
plot(OXY_T2D_plsdaVIPbootbca$VIP, 1:nrow(OXY_T2D_plsdaVIPbootbca), 
	xlim=c(min(OXY_T2D_plsdaVIPbootbca$Lower), max(OXY_T2D_plsdaVIPbootbca$Upper)), 
	axes=FALSE, ylab="", xlab="Variable Importance in Projection", type="n")
with(OXY_T2D_plsdaVIPbootbca, 
	segments(
		Lower, 
		as.numeric(rownames(OXY_T2D_plsdaVIPbootbca)),
		Upper, 
		as.numeric(rownames(OXY_T2D_plsdaVIPbootbca)),
		col="black"
	)
)
points(OXY_T2D_plsdaVIPbootbca$VIP, 1:nrow(OXY_T2D_plsdaVIPbootbca), pch=21, bg=OXY_T2D_plsdaVIPbootbca$Color, 
	col="black", cex=1.5)
axis(2, at=as.numeric(rownames(OXY_T2D_plsdaVIPbootbca)), 
	labels=OXY_T2D_plsdaVIPbootbca$Metabolites, las=2, cex.axis=.9)
axis(1)
box()
abline(v=1, lty=2, col="black")
# dev.off()

## Capture VIPs > 1 for remodeling
OXY_T2D_plsdaVIP <- OXY_T2D_plsdaVIPbootbca[OXY_T2D_plsdaVIPbootbca$VIP > 1,"Metabolites"]

## Fit model
set.seed(1000) # set seed to reproduce results
OXY_T2D_PLSDAcaretVIP <- train(scale(log(OXY_T2D_X[, colnames(OXY_T2D_X) %in% OXY_T2D_plsdaVIP])), # Exploratory variables
	OXY_T2D_dis, # Dependent variable (classifications)
	method = "pls", # algorithm
	tuneLength = 10, # number of latent variables
	trControl = trainControl, # control parameters
	# trControl = trainControl(method="none"), tuneGrid=data.frame(ncomp=5), # Removes CV based selection and forces # of components in model. Replicates results from pls package above.
	# metric = "Accuracy", # model assessment statistic
	preProc = NULL
)
OXY_T2D_PLSDAcaretVIP
# > OXY_T2D_PLSDAcaretVIP
# Partial Least Squares 

# 42 samples
# 19 predictors
 # 2 classes: 'AD', 'ND' 

# No pre-processing
# Resampling: Cross-Validated (10 fold, repeated 10 times) 
# Summary of sample sizes: 38, 37, 38, 37, 38, 38, ... 
# Resampling results across tuning parameters:

  # ncomp  Accuracy   Kappa    
   # 1     0.5831667  0.1586757
   # 2     0.6760000  0.3392924
   # 3     0.6610000  0.3129111
   # 4     0.6920000  0.3791991
   # 5     0.6970000  0.3877423
   # 6     0.6928333  0.3775907
   # 7     0.6513333  0.2922038
   # 8     0.6370000  0.2691036
   # 9     0.6270000  0.2433460
  # 10     0.6260000  0.2410383

# Accuracy was used to select the optimal model using the largest value.
# The final value used for the model was ncomp = 5.

## Identify LV with best CV results
OXY_T2D_PLSDAcaret_VIPbestTune <- unlist(OXY_T2D_PLSDAcaretVIP$bestTune)
## Create data frame for pls function
OXY_T2D_plsdaVIPDF <- data.frame(Y=as.integer(OXY_T2D_dis), X=I(scale(log(OXY_T2D_X[, colnames(OXY_T2D_X) %in% OXY_T2D_plsdaVIP]))))
## PLSDA with tuned LV
OXY_T2DVIPplsda <- pls::mvr(Y~., data=OXY_T2D_plsdaVIPDF, ncomp=OXY_T2D_PLSDAcaret_VIPbestTune, method="oscorespls", validation="CV") 

OXY_T2DVIPplsdaSCORES <- OXY_T2DVIPplsda$scores
attributes(OXY_T2DVIPplsdaSCORES)$class <- NULL
OXY_T2DVIPplsdaSCORES <- as.data.frame(OXY_T2DVIPplsdaSCORES) %>% mutate(label=rownames(OXY_T2DVIPplsdaSCORES)) 
OXY_T2DVIPplsdaSCORES <- inner_join(OXY_T2D[,colnames(OXY_T2D) %in% oxyExperimentalGroups], OXY_T2DVIPplsdaSCORES, by="label")
OXY_T2DVIPplsda_LVvar <- PLSvarAxis(OXY_T2DVIPplsda)

plot3d(OXY_T2DVIPplsdaSCORES[,c("Comp 1","Comp 2","Comp 3")], size=15, box=FALSE, 
	col=ifelse(OXY_T2DVIPplsdaSCORES$Disease %in% "AD", "#66c2a5", "#8da0cb"), 
	xlab=OXY_T2DVIPplsda_LVvar[["YLV1"]],
	ylab=OXY_T2DVIPplsda_LVvar[["YLV2"]],
	zlab=OXY_T2DVIPplsda_LVvar[["YLV3"]])

# rgl.postscript("./Figures/OXY_T2DVIPplsdaSCORES.pdf", "pdf", drawText = TRUE)	

##############################################
## PLSDA in metabolically healthy
##############################################

OXY_MH <- OXY_DATA_T2D_DF[OXY_DATA_T2D_DF$Metab == "MH",]
OXY_MH_X <- OXY_MH[,!(colnames(OXY_MH) %in% oxyExperimentalGroups)]
OXY_MH_dis <- factor(OXY_MH$Disease)

## Caret PLSDA
## Set training function for PLSDA
trainControl <- trainControl(
	method = "repeatedcv", # training method
	number=10, # number of folds
	repeats = 10, # number of complete sets of folds
	classProbs = TRUE, # class probabilities
	summaryFunction = defaultSummary)

## Fit model
set.seed(1000) # set seed to reproduce results
OXY_MH_PLSDAcaret <- train(scale(log(round(OXY_MH_X,2))), # Exploratory variables
	OXY_MH_dis, # Dependent variable (classifications)
	method = "pls", # algorithm
	tuneLength = 10, # number of latent variables
	trControl = trainControl, # control parameters
	preProc = NULL
)
OXY_MH_PLSDAcaret
# > OXY_MH_PLSDAcaret
# Partial Least Squares 

# 84 samples
# 46 predictors
 # 2 classes: 'AD', 'ND' 

# No pre-processing
# Resampling: Cross-Validated (10 fold, repeated 10 times) 
# Summary of sample sizes: 75, 76, 76, 76, 76, 75, ... 
# Resampling results across tuning parameters:

  # ncomp  Accuracy   Kappa     
   # 1     0.5141071  0.01783329
   # 2     0.5839286  0.15213955
   # 3     0.6146825  0.21148180
   # 4     0.5921032  0.16856291
   # 5     0.6037698  0.19521099
   # 6     0.5846032  0.15470230
   # 7     0.5880754  0.16051576
   # 8     0.5935913  0.17378851
   # 9     0.6029762  0.19251514
  # 10     0.6125595  0.21194606

# Accuracy was used to select the optimal model using the largest value.
# The final value used for the model was ncomp = 3.


## Pull out resampling data
OXY_MH_PLSDAcaret_CVres <- OXY_MH_PLSDAcaret$resample
## Make Fold and Repeat column
OXY_MH_PLSDAcaret_CVres$Fold <- sapply(strsplit(OXY_MH_PLSDAcaret_CVres$Resample, "\\."), function(x) x[1])
OXY_MH_PLSDAcaret_CVres$Rep <- sapply(strsplit(OXY_MH_PLSDAcaret_CVres$Resample, "\\."), function(x) x[2])
## order by Repeat and then folds
OXY_MH_PLSDAcaret_CVres <- OXY_MH_PLSDAcaret_CVres[order(OXY_MH_PLSDAcaret_CVres$Rep, OXY_MH_PLSDAcaret_CVres$Fold),]

## Mean Accuracy aross fold - should equal LV with best CV results
mean(OXY_MH_PLSDAcaret_CVres$Accuracy)

## Calculate mean, and sd of Repeats adn then also attach overall mean and sd.
OXY_MH_PLSDAcaret_CVres_MEAN <- OXY_MH_PLSDAcaret_CVres %>% group_by(Rep) %>% 
	summarise(MEAN=mean(100*Accuracy), SD=sd(100*Accuracy)) %>%
	bind_rows(data.frame(Rep="All", MEAN=mean(100*OXY_MH_PLSDAcaret_CVres$Accuracy), SD=sd(100*OXY_MH_PLSDAcaret_CVres$Accuracy))) %>%
	as.data.frame()
## Set row names as Repeat labels
rownames(OXY_MH_PLSDAcaret_CVres_MEAN) <- OXY_MH_PLSDAcaret_CVres_MEAN$Rep
## Remove Rep column
OXY_MH_PLSDAcaret_CVres_MEAN$Rep <- NULL
## Calculate upper and lower mean+sd
OXY_MH_PLSDAcaret_CVres_MEAN$Upper <- with(OXY_MH_PLSDAcaret_CVres_MEAN, MEAN+SD)
OXY_MH_PLSDAcaret_CVres_MEAN$Lower <- with(OXY_MH_PLSDAcaret_CVres_MEAN, MEAN-SD)

## Plot mean CV results across each LV and mean CV across repeats of best LV
# pdf("./Figures/MH PLSDA Caret CV results 71519.pdf", height=12, width=7, useDingbats=FALSE)	
par(mfrow=c(2,1))
plot(OXY_MH_PLSDAcaret$results$ncomp, 100*OXY_MH_PLSDAcaret$results$Accuracy, type="l", pch=21, bg="black", ylim=c(0,100),
	ylab="Average Cross-Validation Accuracy, %", xlab="Latent Variable")
OXY_MH_PLSDAcaret_CVres_MEAN_bp <- barplot(as.matrix(OXY_MH_PLSDAcaret_CVres_MEAN$MEAN), beside=TRUE, ylim=c(0, max(as.matrix(OXY_MH_PLSDAcaret_CVres_MEAN))),
	names.arg=rownames(OXY_MH_PLSDAcaret_CVres_MEAN), col=c(rep("#FFFFFF", 10), "gray"), ylab="Average Cross-Validation Accuracy, %")
segments(OXY_MH_PLSDAcaret_CVres_MEAN_bp, OXY_MH_PLSDAcaret_CVres_MEAN$Upper, OXY_MH_PLSDAcaret_CVres_MEAN_bp, OXY_MH_PLSDAcaret_CVres_MEAN$Lower)
# dev.off()


## Identify LV with best CV results
OXY_MH_PLSDAcaret_bestTune <- unlist(OXY_MH_PLSDAcaret$bestTune)
## Create data frame for pls function
OXY_MH_plsdaDF <- data.frame(Y=as.integer(OXY_MH_dis), X=I(scale(log(OXY_MH_X))))
## PLSDA with tuned LV
OXY_MHplsda <- pls::mvr(Y~., data=OXY_MH_plsdaDF, ncomp=OXY_MH_PLSDAcaret_bestTune, method="oscorespls", validation="CV") 
## Boostrapped VIP 
OXY_MH_plsdaVIPboot <- plsdaVIPboot_FUN(OXY_MH_plsdaDF, nc=OXY_MH_PLSDAcaret_bestTune, iter=500, Ytype="Categorical")

## Use Bootstrapped BCA CI
OXY_MH_plsdaVIPbootbca <- OXY_MH_plsdaVIPboot$BOOTCI[["bca"]]
## Make Metabolites column
OXY_MH_plsdaVIPbootbca$Metabolites <- rownames(OXY_MH_plsdaVIPbootbca)
## Make data frame with Metabolites and VIP calculations
OXY_MHplsda_VIPDF <- data.frame(Metabolites=names(colMeans(VIP(OXY_MHplsda)[1:OXY_MH_PLSDAcaret_bestTune,])), 
	VIP=colMeans(VIP(OXY_MHplsda)[1:OXY_MH_PLSDAcaret_bestTune,]))
## Join VIPs with boostrapped VIP CIs
OXY_MH_plsdaVIPbootbca <- full_join(OXY_MHplsda_VIPDF, OXY_MH_plsdaVIPbootbca, by="Metabolites") %>%
	arrange(VIP) %>%
	mutate(Color = ifelse(VIP > 1, "black", "grey"), )

## How many VIP > 1
sum(OXY_MH_plsdaVIPbootbca$VIP > 1)
# > sum(OXY_MH_plsdaVIPbootbca$VIP > 1) 
# [1] 12

## Plot VIP and bootstrapped VIP CIs
# pdf("./Figures/MH PLSDA VIP Plot 71519.pdf", height=10, width=6, useDingbats=FALSE)
par(mfrow=c(1,1))
plot(OXY_MH_plsdaVIPbootbca$VIP, 1:nrow(OXY_MH_plsdaVIPbootbca), 
	xlim=c(min(OXY_MH_plsdaVIPbootbca$Lower), max(OXY_MH_plsdaVIPbootbca$Upper)), 
	axes=FALSE, ylab="", xlab="Variable Importance in Projection", type="n")
with(OXY_MH_plsdaVIPbootbca, 
	segments(
		Lower, 
		as.numeric(rownames(OXY_MH_plsdaVIPbootbca)),
		Upper, 
		as.numeric(rownames(OXY_MH_plsdaVIPbootbca)),
		col="black"
	)
)
points(OXY_MH_plsdaVIPbootbca$VIP, 1:nrow(OXY_MH_plsdaVIPbootbca), pch=21, bg=OXY_MH_plsdaVIPbootbca$Color, 
	col="black", cex=1.5)
axis(2, at=as.numeric(rownames(OXY_MH_plsdaVIPbootbca)), 
	labels=OXY_MH_plsdaVIPbootbca$Metabolites, las=2, cex.axis=.9)
axis(1)
box()
abline(v=1, lty=2, col="black")
# dev.off()

## Capture VIPs > 1 for remodeling
OXY_MH_plsdaVIP <- OXY_MH_plsdaVIPbootbca[OXY_MH_plsdaVIPbootbca$VIP > 1,"Metabolites"]

## Fit model
set.seed(1000) # set seed to reproduce results
OXY_MH_PLSDAcaretVIP <- train(scale(log(OXY_MH_X[, colnames(OXY_MH_X) %in% OXY_MH_plsdaVIP])), # Exploratory variables
	OXY_MH_dis, # Dependent variable (classifications)
	method = "pls", # algorithm
	tuneLength = 10, # number of latent variables
	trControl = trainControl, # control parameters
	# trControl = trainControl(method="none"), tuneGrid=data.frame(ncomp=5), # Removes CV based selection and forces # of components in model. Replicates results from pls package above.
	# metric = "Accuracy", # model assessment statistic
	preProc = NULL
)
OXY_MH_PLSDAcaretVIP
# > OXY_MH_PLSDAcaretVIP 
# Partial Least Squares 

# 84 samples
# 12 predictors
 # 2 classes: 'AD', 'ND' 

# No pre-processing
# Resampling: Cross-Validated (10 fold, repeated 10 times) 
# Summary of sample sizes: 75, 76, 76, 76, 76, 75, ... 
# Resampling results across tuning parameters:

  # ncomp  Accuracy   Kappa    
   # 1     0.6895833  0.3697433
   # 2     0.6377579  0.2567663
   # 3     0.6496032  0.2818169
   # 4     0.6483532  0.2795610
   # 5     0.6488095  0.2805458
   # 6     0.6392262  0.2633824
   # 7     0.6296032  0.2447575
   # 8     0.6294643  0.2446113
   # 9     0.6268254  0.2395034
  # 10     0.6282143  0.2411708

# Accuracy was used to select the optimal model using the largest value.
# The final value used for the model was ncomp = 1.


## Identify LV with best CV results
OXY_MH_PLSDAcaret_VIPbestTune <- unlist(OXY_MH_PLSDAcaret$bestTune)
## Create data frame for pls function
OXY_MH_plsdaVIPDF <- data.frame(Y=as.integer(OXY_MH_dis), X=I(scale(log(OXY_MH_X[, colnames(OXY_MH_X) %in% OXY_MH_plsdaVIP]))))
## PLSDA with tuned LV
OXY_MHVIPplsda <- pls::mvr(Y~., data=OXY_MH_plsdaVIPDF, ncomp=OXY_MH_PLSDAcaret_VIPbestTune, method="oscorespls", validation="CV") 

OXY_MHVIPplsdaSCORES <- OXY_MHVIPplsda$scores
attributes(OXY_MHVIPplsdaSCORES)$class <- NULL
OXY_MHVIPplsdaSCORES <- as.data.frame(OXY_MHVIPplsdaSCORES) %>% mutate(label=rownames(OXY_MHVIPplsdaSCORES)) 
OXY_MHVIPplsdaSCORES <- inner_join(OXY_MH[,colnames(OXY_MH) %in% oxyExperimentalGroups], OXY_MHVIPplsdaSCORES, by="label")
OXY_MHVIPplsda_LVvar <- PLSvarAxis(OXY_MHVIPplsda)

plot3d(OXY_MHVIPplsdaSCORES[,c("Comp 1","Comp 2","Comp 3")], size=15, box=FALSE, 
	col=ifelse(OXY_MHVIPplsdaSCORES$Disease %in% "AD", "#66c2a5", "#8da0cb"), 
	xlab=OXY_MHVIPplsda_LVvar[["YLV1"]],
	ylab=OXY_MHVIPplsda_LVvar[["YLV2"]],
	zlab=OXY_MHVIPplsda_LVvar[["YLV3"]])

# rgl.postscript("./Figures/OXY_MHVIPplsdaSCORES_3D.pdf", "pdf", drawText = TRUE)	


# > sessionInfo()
# R version 3.6.0 (2019-04-26)
# Platform: x86_64-w64-mingw32/x64 (64-bit)
# Running under: Windows 7 x64 (build 7601) Service Pack 1

# Matrix products: default

# locale:
# [1] LC_COLLATE=English_United States.1252  LC_CTYPE=English_United States.1252    LC_MONETARY=English_United States.1252
# [4] LC_NUMERIC=C                           LC_TIME=English_United States.1252    

# attached base packages:
# [1] stats     graphics  grDevices utils     datasets  methods   base     

# other attached packages:
 # [1] flextable_0.5.5 boot_1.3-22     caret_6.0-84    lattice_0.20-38 pls_2.7-1       car_3.0-3       carData_3.0-2   impute_1.58.0  
 # [9] outliers_0.14   forcats_0.4.0   stringr_1.4.0   dplyr_0.8.1     purrr_0.3.2     readr_1.3.1     tidyr_0.8.3     tibble_2.1.3   
# [17] ggplot2_3.2.0   tidyverse_1.2.1 readxl_1.3.1   

# loaded via a namespace (and not attached):
 # [1] nlme_3.1-139            lubridate_1.7.4         webshot_0.5.1           httr_1.4.0              tools_3.6.0            
 # [6] backports_1.1.4         R6_2.4.0                rpart_4.1-15            lazyeval_0.2.2          colorspace_1.4-1       
# [11] manipulateWidget_0.10.0 nnet_7.3-12             withr_2.1.2             tidyselect_0.2.5        curl_3.3               
# [16] compiler_3.6.0          cli_1.1.0               rvest_0.3.4             xml2_1.2.0              officer_0.3.5          
# [21] scales_1.0.0            digest_0.6.19           foreign_0.8-71          rmarkdown_1.13          rio_0.5.16             
# [26] base64enc_0.1-3         pkgconfig_2.0.2         htmltools_0.3.6         htmlwidgets_1.3         rlang_0.3.4            
# [31] rstudioapi_0.10         shiny_1.3.2             generics_0.0.2          jsonlite_1.6            crosstalk_1.0.0        
# [36] ModelMetrics_1.2.2      zip_2.0.2               magrittr_1.5            Matrix_1.2-17           Rcpp_1.0.1             
# [41] munsell_0.5.0           abind_1.4-5             gdtools_0.1.9           stringi_1.4.3           MASS_7.3-51.4          
# [46] plyr_1.8.4              recipes_0.1.5           grid_3.6.0              promises_1.0.1          crayon_1.3.4           
# [51] miniUI_0.1.1.1          haven_2.1.0             splines_3.6.0           hms_0.4.2               knitr_1.23             
# [56] pillar_1.4.1            uuid_0.1-2              reshape2_1.4.3          codetools_0.2-16        stats4_3.6.0           
# [61] glue_1.3.1              evaluate_0.14           data.table_1.12.2       modelr_0.1.4            httpuv_1.5.1           
# [66] foreach_1.4.4           cellranger_1.1.0        gtable_0.3.0            assertthat_0.2.1        xfun_0.7               
# [71] gower_0.2.1             openxlsx_4.1.0.1        mime_0.7                prodlim_2018.04.18      xtable_1.8-4           
# [76] broom_0.5.2             e1071_1.7-2             later_0.8.0             class_7.3-15            survival_2.44-1.1      
# [81] timeDate_3043.102       iterators_1.0.10        lava_1.6.5              rgl_0.100.19            ipred_0.9-9            







